/* Zodiac - Main.cpp
 * Entry point to the game.
 */

#include "DisplayManager.h"
#include "ModelViewer.h"
#include <SDL.h>
#include <SDL_opengl.h>

int width = 640, height = 480;
bool fullscreen = false;
char *logFile = NULL;

void usage()
{
	printf("Usage: ModelViewer.exe [options]\n");
	printf("  -w           Specify a width for the viewer (e.g. -w 800)\n");
	printf("  -h           Specify a height for the viewer (e.g. -h 600)\n");
	printf("  -fullscreen  Make the viewer run in fullscreen mode.\n");
	printf("  -log         Log activity to a file. (e.g. -log log.txt)\n");
}

void checkOptions(int argc, char **argv)
{
	for (int i = 0;i < argc; ++i)
	{
		if (strcmp("-fullscreen", argv[i]) == 0)
		{
			fullscreen = true;
		} else if (strcmp("-log", argv[i]) == 0)
		{
			// The argument after "-log" should be a file to log to
			i++;
			logFile = argv[i];
		} else if (strcmp("-w", argv[i]) == 0)
		{
			// The argument after "-w" should be a width
			i++;
			int c = atoi(argv[i]);
			width = (width > c) ? width : c;
		} else if (strcmp("-h", argv[i]) == 0)
		{
			// The argument after "-h" should be a height
			i++;
			int c = atoi(argv[i]);
			height = (height > c) ? height : c;
		} else if (strcmp("-?", argv[i]) == 0)
		{
			// The user asked for help; print that out and leave
			usage();
			exit(0);
		}
	}
}

int SDL_main(int argc, char **argv)
{
	checkOptions(argc, argv);

	DisplayManager display;
	if (!display.openDisplay(width, height, 32, fullscreen))
	{
		return -1;
	}

	ModelViewer viewer;
	return viewer.run(display.displayWidth(), display.displayHeight(), logFile);
}
