/* Zodiac - Md2Model.h
 * A Class for loading/rendering Quake 2 and Egoboo md2 models.
 */

#ifndef Zodiac_Md2Model
#define Zodiac_Md2Model

struct Md2Vertex
{
	float x, y, z;
	unsigned normal;	// index to id-normal array
};

struct Md2TexCoord
{
	float s, t;
};

struct Md2Triangle
{
	short vertexIndices[3];
	short texCoordIndices[3];
};

struct Md2Frame
{
	char name[16];
	float min[3], max[3];		// axis-aligned bounding box limits
	Md2Vertex *vertices;
};

struct Md2SkinName
{
	char name[64];
};

class Md2Model
{
private:
	int m_numVertices;
	int m_numTexCoords;
	int m_numTriangles;
	int m_numSkins;
	int m_numFrames;

	Md2SkinName *m_skins;
	Md2TexCoord *m_texCoords;
	Md2Triangle *m_triangles;
	Md2Frame    *m_frames;

	void blendFrames(int frame1, int frame2, float lerp) const;
	Md2Model();

public:
	~Md2Model();
	static Md2Model* loadFromFile(const char *fileName);

	int numVertices() const { return m_numVertices; }
	int numTexCoords() const { return m_numTexCoords; }
	int numTriangles() const { return m_numTriangles; }
	int numSkins() const { return m_numSkins; }
	int numFrames() const { return m_numFrames; }

	const Md2SkinName *getSkin(int index) const;
	const Md2Frame *getFrame(int index) const;
	const Md2TexCoord *getTexCoords() const { return m_texCoords; }
	const Md2Triangle *getTriangles() const { return m_triangles; }

	void drawFrame(int frame) const;
	void drawBlendedFrames(int frame1, int frame2, float lerp) const;
};

#endif // include guard
