/* Zodiac - Texture.cpp
 */

#include "Texture.h"
#include <SDL.h>
#include <SDL_opengl.h>

static int powerOfTwo(int input)
{
	int value = 1;

	while(value < input)
	{
		value <<= 1;
	}
	return value;
}

Texture *Texture::load(const char *fileName)
{
	int w, h;
	SDL_Surface *source, *image;
	SDL_Rect area;
	Texture *texture;

	// Load the texture from disk
	source = SDL_LoadBMP(fileName);
	if (source == NULL)
	{
		return NULL;
	}

	// Use the surface width & height expanded to the next powers of two
	w = powerOfTwo(source->w);
	h = powerOfTwo(source->h);

#if SDL_BYTEORDER == SDL_LIL_ENDIAN
	image = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, 0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000);
#else
	image = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
#endif

	if( image == NULL)
	{
		SDL_FreeSurface(source);
		return NULL;
	}

	// Turn off source alpha on this image if it's on
	if ( (source->flags & SDL_SRCALPHA) == SDL_SRCALPHA )
	{
		SDL_SetAlpha(source, 0, 0);
	}

	// Copy the surface into the texture image
	area.x = 0;
	area.y = 0;
	area.w = source->w;
	area.h = source->h;
	SDL_BlitSurface(source, &area, image, &area);

	// Don't need the extra image anymore
	SDL_FreeSurface(source);

	// Allocate a texture for it
	texture = new Texture();
	glGenTextures(1, &texture->m_binding);

	// Save the new width/height and the source surface
	texture->m_width = w;
	texture->m_height = h;
	texture->m_srcWidth = area.w;
	texture->m_srcHeight = area.h;
	texture->m_surface = image;

	// Send the texture to OpenGL
	texture->upload();

	return texture;
}

Texture::Texture()
{
	m_binding = 0;
	m_width = m_height = 0;
	m_surface = NULL;
}

Texture::~Texture()
{
	if (m_binding != 0)
	{
		glDeleteTextures(1, &m_binding);
	}

	if (m_surface != NULL)
	{
		SDL_FreeSurface(m_surface);
	}
}

Texture* Texture::loadFromFile(const char *fileName)
{
	// ignore garbage input
	if (!fileName || !fileName[0]) return NULL;

	Texture *t = load(fileName);
	if (t == NULL)
	{
		// no luck
		return NULL;
	}

	return t;
}

void Texture::upload()
{
	glBindTexture(GL_TEXTURE_2D, m_binding);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D,	0, GL_RGBA,	m_width, m_height, 0,
		GL_RGBA, GL_UNSIGNED_BYTE, m_surface->pixels);
}
