/* Zodiac - UiContext.h
 */

#ifndef Zodiac_UiContext
#define Zodiac_UiContext

#include <SDL.h>
#include "Font.h"
#include "Texture.h"

typedef unsigned int UI_ID;
#define UI_Nothing (UI_ID)(-1)

struct UiRect {
	int x, y, width, height;

	UiRect(int x_, int y_, int width_, int height_) {
		x = x_;
		y = y_;
		width = width_;
		height = height_;
	}
};

struct UiPoint {
	int x, y;

	UiPoint(int x_ = 0, int y_ = 0)
	{
		x = x_;
		y = y_;
	}
};

class UiContext
{
private:
	UI_ID m_activeControl;
	UI_ID m_hotControl;

	// Mouse status
	UiPoint m_mousePosition;
	UiPoint m_mouseLastPosition;
	bool m_mousePressed;
	bool m_mouseReleased;
	bool m_mouseBeingDragged;

	Font *m_font;

public:
	UiContext();
	~UiContext();

	void setFont(Font *font);
	Font *font() { return m_font; }

	// Anything useful done w/ the Ui needs to be wrapped in calls to
	// beginFrame and endFrame
	void beginFrame(float deltaTime);
	void endFrame();

	void handleSDLEvent(SDL_Event *evt);

	void setActiveControl(UI_ID control);
	void setHotControl(UI_ID control);

	UI_ID activeControl() const;
	UI_ID hotControl() const;
	bool  mousePressed() const { return m_mousePressed; }
	bool  mouseReleased() const { return m_mouseReleased; }
	bool  mouseBeingDragged() const { return m_mouseBeingDragged; }
	
	UiPoint mousePosition() const { return m_mousePosition; }
	UiPoint mouseDragDistance() const;

	bool mouseInsideRect(UiRect rect) const;

	int getScreenWidth() const;
	int getScreenHeight() const;

	// Controls
	bool doButton(UI_ID id, const char *label, UiRect rect);
	
	// Control logic (separated out to make it easier to re-use)
	bool buttonLogic(UI_ID id, UiRect rect);

	// Drawing functions
	void drawButton(UiRect rect, bool active, bool hot) const;
	void drawImage(Texture *img, UiRect rect) const;
};

#endif // include guard
